/**
 * this class represents a class which contains a method to load data from RequestData class and set the ResponseData class fields
 */
package com.insomnia.parser;
import javax.net.ssl.HttpsURLConnection;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashMap;

public class RequestLoader {
    /**
     * a method to load request data and get response data
     * @param data is the information we get from request
     *
     */
    public static ResponseData run (RequestData data){
        String boundary = "*****" + System.currentTimeMillis() + "*****";
        ResponseData responseData = new ResponseData(data);

        if (data.getUrl().isEmpty()){
            responseData.setSyntaxError(true);
            responseData.setSyntaxErrorLog("enter url");
            return responseData.finish();
        }


        File file = null;

        if (!data.getUploadPath().isEmpty()){
            file = new File(data.getUploadPath());
            if (!file.exists()){
                responseData.setSyntaxError(true);
                responseData.setSyntaxErrorLog("enter a valid upload path");
                return responseData.finish();
            }
        }

        try {
            String mUrl = data.getUrl();
            if (data.getMethod().equals("GET")) {
                if (mUrl.contains("?")) {
                    mUrl = mUrl + "&" + data.getBody();
                }else{
                    mUrl = mUrl + "?" + data.getBody();
                }
            }

            URL url = new URL(mUrl);
            HttpURLConnection con = (HttpURLConnection) url.openConnection();

            con.setUseCaches(false);
            if (!data.getMethod().equals("GET") && (!data.getBody().isEmpty()||!data.getUploadPath().isEmpty()) ){
                con.setDoOutput(true);
            }

            con.setDoInput(true);
            con.setInstanceFollowRedirects(data.isFollowRedirects());//change direction

            con.setRequestMethod(data.getMethod());
            String content_type = "";
            if (data.getHeaders()!=null){
                for (String key : data.getHeaders().keySet()){
                    if (key.toLowerCase().equals("content-type")) {
                        content_type = data.getHeaders().get(key);
                        continue;
                    }
                    con.setRequestProperty(key,data.getHeaders().get(key));
                }
            }
            if (data.isJson()){
                content_type = "application/json; charset=UTF-8";
            }
            con.setRequestProperty("Connection", "Keep-Alive");
            con.setRequestProperty("Cache-Control", "no-cache");

            if (con.getDoOutput()){

                if (content_type.toLowerCase().equals("multipart/form-data")) {
                    con.setRequestProperty(
                            "Content-Type", "multipart/form-data;boundary=" + boundary);
                    responseData.getRequestData().isMultipart = true;
                }else{
                    con.setRequestProperty("Content-Type",content_type);
                }
                if (content_type.toLowerCase().equals("multipart/form-data") || !data.getUploadPath().isEmpty()) {


                    DataOutputStream request = new DataOutputStream(con.getOutputStream());
                    String twoHyphens = "--";
                    String crlf = "\r\n";
                    if (!data.getBody().isEmpty()) {
                        HashMap<String, String> fields = new HashMap<>();
                        if (!data.getBody().contains("&")) {
                            String[] f1 = data.getBody().split("=");
                            fields.put(f1[0], f1[1]);
                        } else {
                            String[] mFields = data.getBody().split("&");
                            for (String f : mFields) {
                                String[] f0 = f.split("=");
                                fields.put(f0[0], f0[1]);
                            }
                        }

                        for (String key : fields.keySet()) {
                            request.writeBytes(twoHyphens + boundary + crlf);
                            request.writeBytes("Content-Disposition: form-data; name=\"" + key + "\"" + crlf);
                            request.writeBytes("Content-Type: text/plain; charset=UTF-8" + crlf);
                            request.writeBytes(crlf);
                            request.writeBytes(fields.get(key) + crlf);
                            request.flush();
                        }
                    }

                    if (!data.getUploadPath().isEmpty()) {
                        String fieldName = "file";

                        String fileName = file.getName();
                        request.writeBytes(twoHyphens + boundary + crlf);
                        request.writeBytes("Content-Disposition: form-data; name=\"" +
                                fieldName + "\";filename=\"" +
                                fileName + "\"" + crlf);
                        request.writeBytes(crlf);


                        FileInputStream fileInputStream = new FileInputStream(file);
                        int bytesRead, bytesAvailable, bufferSize;
                        byte[] buffer;
                        int maxBufferSize = 1 * 1024 * 1024;
                        bytesAvailable = fileInputStream.available();
                        bufferSize = Math.min(bytesAvailable, maxBufferSize);
                        buffer = new byte[bufferSize];
                        bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                        while (bytesRead > 0) {
                            request.write(buffer, 0, bufferSize);
                            bytesAvailable = fileInputStream.available();
                            bufferSize = Math.min(bytesAvailable, maxBufferSize);
                            bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                        }

                        request.writeBytes(crlf);
                    }

                    request.writeBytes(twoHyphens + boundary +
                            twoHyphens + crlf);

                    request.flush();

                    request.close();
                }else{
                    OutputStream os = con.getOutputStream();
                    os.write(data.getBody().getBytes("UTF-8"));
                    os.close();
                }

            }             else{
            con.setRequestProperty("Content-Type",content_type);
        }

            int status = con.getResponseCode();
            responseData.setStatus(status);
            if (status == HttpURLConnection.HTTP_OK || status == HttpURLConnection.HTTP_ACCEPTED) {
                InputStream responseStream = new
                        BufferedInputStream(con.getInputStream());
                responseData.setResponseHeaders(con.getHeaderFields());
                responseData.setSuccessful(true);

                //Save files
                if (responseData.getRequestData().getOutput().equals("-null")) {
                    File f = new File(System.getProperty("java.io.tmpdir"), "insomnia");
                    try {
                        FileOutputStream outputStream = new FileOutputStream(f);
                        int read;
                        byte[] bytes = new byte[1024];

                        while ((read = responseStream.read(bytes)) != -1) {
                            outputStream.write(bytes, 0, read);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    responseData.setSize(f.length());
                    responseData.res = new String(Files.readAllBytes(Paths.get(f.getAbsolutePath())), Charset.forName("utf-8"));
                    f.delete();
                } else {
                    Saver.saveResponse(responseData, responseStream);
                }
                if (data.isSaveRequest()) Saver.saveRequest(data);


                con.disconnect();

            }else if (status == HttpsURLConnection.HTTP_NO_CONTENT){
                responseData.setSuccessful(true);
                responseData.res = "No body returned for response";
                con.disconnect();
            } else {
                responseData.setSuccessful(false);

                if (con.getErrorStream()!=null) {
                    BufferedReader reader = new BufferedReader(new InputStreamReader((con.getErrorStream())));
                    StringBuilder builder = new StringBuilder();
                    String line;
                    while ((line = reader.readLine()) != null) {
                        builder.append(line);
                    }
                    responseData.setErrorMessage(builder.toString());
                }
            }

        } catch (MalformedURLException e) {
            responseData.setSyntaxError(true);
            responseData.setSyntaxErrorLog("enter a valid url");
            return responseData.finish();

        } catch (IOException e) {
            responseData.setSuccessful(false);
            responseData.setErrorMessage(e.getMessage());
        }

        return responseData.finish();
    }
}
