/**
 * represents a class that contains some methods to save requests
 * @zahra_fatehi
 * @version 0.0
 */
package com.insomnia.parser;
import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
public class Saver {
    private static final String appName = "Insomnia";

    /**
     * get the direction app(insomnia)
     * @return the the direction of he app as string
     */
    public static String getDirApp(){
        return System.getProperty("user.dir");
    }

    /**
     * get the outputs of requests direction
     * @return the address of where we save requests
     */
    public static String getOutputDir(){
        File f = new File(getDirApp(),"Outputs");
        f.mkdir();
        return f.toString();
    }

    /**
     *
     * @return
     */
    public static String getRequestsDir(){
        File f = new File(getDirApp(),"Requests");
        f.mkdir();
        return f.toString();
    }

    /**
     * @return saved file name or empty if file didn't save
     */
    public static String saveRequest(RequestData data) {
        int size = new File(getRequestsDir()).list().length + 1;
        String fileName = "Request "+size+".insomnia";
        File file = new File(getRequestsDir(),fileName);
        boolean fileFound = file.exists();
        if (fileFound) {
            while (fileFound) {
                size++;
                fileName = "Request "+size+".insomnia";
                file = new File(getRequestsDir(),fileName);
                fileFound = file.exists();
            }
        }

        try {
            file.createNewFile();
            FileOutputStream fo = new FileOutputStream(file);
            ObjectOutputStream out = new ObjectOutputStream(fo);
            out.writeObject(data);
            out.close();
            fo.close();
            return fileName;
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * this method saves the requests and their responses
     * @param data is the request data
     */
    public void saveRequestAndResponse (ResponseData data) {

        String fileName = data.getRequestData().getName()+".insomnia";
        File file = new File(getRequestsDir(),fileName);

        try {
            file.createNewFile();
            FileOutputStream fo = new FileOutputStream(file);
            ObjectOutputStream out = new ObjectOutputStream(fo);
            out.writeObject(data);
            out.close();
            fo.close();

        } catch (IOException e) {
            e.printStackTrace();

        }
    }

    /**
     * this method is for read responses from a file
     * @param fileName is the name of the file where requests and responses will be save there
     * @return the response data
     */
    public ResponseData readRequestAndResponse (String fileName){
        try {
            FileInputStream file = new FileInputStream(new File(getRequestsDir(),fileName+".insomnia"));
            ObjectInputStream in = new ObjectInputStream(file);

            ResponseData data = (ResponseData) in.readObject();

            in.close();
            file.close();
            return  data;
        } catch(IOException | ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * this method is for saving responses in a file
     * @param data is the response data
     * @param stream
     */
    public static void saveResponse(ResponseData data, InputStream stream) {
        if (!data.getRequestData().getOutput().equals("-null")){
            String fileName;
            if (data.getRequestData().getOutput().equals("-t")){
                fileName = "output_"+new SimpleDateFormat("dd_MM_yyyy").format(new Date());
            }else{
                fileName = data.getRequestData().getOutput();
            }

            File file = new File(getOutputDir(),fileName);
            try{
                FileOutputStream outputStream = new FileOutputStream(file);
                int read;
                byte[] bytes = new byte[1024];

                while ((read = stream.read(bytes)) != -1) {
                    outputStream.write(bytes, 0, read);
                }

                data.setSize(file.length());
                data.res = new String(Files.readAllBytes(Paths.get(file.getAbsolutePath())), Charset.forName("utf-8"));
            }catch (IOException e) {
                e.printStackTrace();
            }

        }
    }

    /**
     * this method is for getting files
     * @return the files of requests
     */
    public static File[] getRequestFiles(){
        return new File(getRequestsDir()).listFiles();
    }

}
